@@ -45,7 +45,7 @@ diskGeo DiskGeometryList[] = {
 	{0, 0, 0, 0, 0}
 };
 
-Bitu call_int13;
+callback_number_t call_int13 = 0;
 Bitu diskparm0, diskparm1;
 static uint8_t last_status;
 static uint8_t last_drive;
@@ -380,7 +380,7 @@ static Bitu INT13_DiskHandler(void) {
 			CALLBACK_SCF(true);
 			return CBRET_NONE;
 		}
-		if (drivenum >= MAX_DISK_IMAGES || imageDiskList[drivenum] == nullptr) {
+		if (drivenum >= MAX_DISK_IMAGES || !imageDiskList[drivenum]) {
 			if (drivenum >= DOS_DRIVES || !Drives[drivenum] || Drives[drivenum]->isRemovable()) {
 				reg_ah = 0x01;
 				CALLBACK_SCF(true);
