@@ -1,5 +1,5 @@
-
 /*
+ *  Copyright (C) 2021-2022  The DOSBox Staging Team
  *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "../dos/program_more_output.h"
 #include "callback.h"
 #include "control.h"
 #include "fs_utils.h"
@@ -34,7 +35,7 @@
 #include "support.h"
 #include "timer.h"
 
-Bitu call_shellstop;
+callback_number_t call_shellstop = 0;
 /* Larger scope so shell_del autoexec can use it to
  * remove things from the environment */
 DOS_Shell *first_shell = nullptr;
@@ -54,6 +55,11 @@ typedef std::list<std::string>::iterator auto_it;
 
 void VFILE_Remove(const char *name, const char *dir = "");
 
+AutoexecObject::AutoexecObject(const std::string& line)
+{
+	Install(line);
+}
+
 void AutoexecObject::Install(const std::string &in) {
 	if (GCC_UNLIKELY(installed))
 		E_Exit("autoexec: already created %s", buf.c_str());
@@ -486,7 +492,9 @@ void DOS_Shell::Run()
 	char input_line[CMD_MAXLINE] = {0};
 	std::string line;
 	if (cmd->FindExist("/?", false) || cmd->FindExist("-?", false)) {
-		WriteOut(MSG_Get("SHELL_CMD_COMMAND_HELP_LONG"));
+		MoreOutputStrings output(*this);
+		output.AddString(MSG_Get("SHELL_CMD_COMMAND_HELP_LONG"));
+		output.Display();
 		return;
 	}
 	if (cmd->FindStringRemainBegin("/C",line)) {
@@ -935,7 +943,7 @@ static char const * const init_line="/INIT AUTOEXEC.BAT";
 void SHELL_Init() {
 	/* Add messages */
 	MSG_Add("SHELL_ILLEGAL_PATH","Illegal Path.\n");
-	MSG_Add("SHELL_CMD_HELP","If you want a list of all supported commands type \033[33;1mhelp /all\033[0m .\nA short list of the most often used commands:\n");
+	MSG_Add("SHELL_CMD_HELP","If you want a list of all supported commands type [color=yellow]help /all[reset] .\nA short list of the most often used commands:\n");
 	MSG_Add("SHELL_CMD_COMMAND_HELP_LONG",
 	        "Starts the DOSBox Staging command shell.\n"
 	        "Usage:\n"
@@ -961,9 +969,9 @@ void SHELL_Init() {
 	MSG_Add("SHELL_ILLEGAL_SWITCH","Illegal switch: %s.\n");
 	MSG_Add("SHELL_MISSING_PARAMETER","Required parameter missing.\n");
 	MSG_Add("SHELL_CMD_CHDIR_ERROR","Unable to change to: %s.\n");
-	MSG_Add("SHELL_CMD_CHDIR_HINT","Hint: To change to different drive type \033[31m%c:\033[0m\n");
-	MSG_Add("SHELL_CMD_CHDIR_HINT_2","directoryname is longer than 8 characters and/or contains spaces.\nTry \033[31mcd %s\033[0m\n");
-	MSG_Add("SHELL_CMD_CHDIR_HINT_3","You are still on drive Z:, change to a mounted drive with \033[31mC:\033[0m.\n");
+	MSG_Add("SHELL_CMD_CHDIR_HINT","Hint: To change to different drive type [color=light-red]%c:[reset]\n");
+	MSG_Add("SHELL_CMD_CHDIR_HINT_2","directoryname is longer than 8 characters and/or contains spaces.\nTry [color=light-red]cd %s[reset]\n");
+	MSG_Add("SHELL_CMD_CHDIR_HINT_3","You are still on drive Z:, change to a mounted drive with [color=light-red]C:[reset].\n");
 	MSG_Add("SHELL_CMD_DATE_HELP", "Displays or changes the internal date.\n");
 	MSG_Add("SHELL_CMD_DATE_ERROR", "The specified date is not correct.\n");
 	MSG_Add("SHELL_CMD_DATE_DAYS", "3SunMonTueWedThuFriSat"); // "2SoMoDiMiDoFrSa"
@@ -1031,7 +1039,7 @@ void SHELL_Init() {
 	MSG_Add("SHELL_CMD_DIR_INTRO"," Directory of %s\n");
 	MSG_Add("SHELL_CMD_DIR_BYTES_USED","%17d file(s) %21s bytes\n");
 	MSG_Add("SHELL_CMD_DIR_BYTES_FREE","%17d dir(s)  %21s bytes free\n");
-	MSG_Add("SHELL_EXECUTE_DRIVE_NOT_FOUND","Drive %c does not exist!\nYou must \033[31mmount\033[0m it first. Type \033[1;33mintro\033[0m or \033[1;33mintro mount\033[0m for more information.\n");
+	MSG_Add("SHELL_EXECUTE_DRIVE_NOT_FOUND","Drive %c does not exist!\nYou must [color=light-red]mount[reset] it first. Type [color=yellow]intro[reset] or [color=yellow]intro mount[reset] for more information.\n");
 	MSG_Add("SHELL_EXECUTE_ILLEGAL_COMMAND","Illegal command: %s.\n");
 	MSG_Add("SHELL_CMD_PAUSE", "Press a key to continue...");
 	MSG_Add("SHELL_CMD_PAUSE_HELP", "Waits for a keystroke to continue.\n");
@@ -1055,40 +1063,36 @@ void SHELL_Init() {
 	MSG_Add("SHELL_CMD_SUBST_FAILURE","SUBST failed. You either made an error in your commandline or the target drive is already used.\nIt's only possible to use SUBST on Local drives");
 
 	MSG_Add("SHELL_STARTUP_BEGIN",
-	        "\033[44;1m\xC9\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD"
-	        "\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD"
-	        "\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xBB\n"
-	        "\xBA \033[32mWelcome to DOSBox Staging %-40s\033[37m \xBA\n"
-	        "\xBA                                                                    \xBA\n"
-	        "\xBA For a short introduction for new users type: \033[33mINTRO\033[37m                 \xBA\n"
-	        "\xBA For supported shell commands type: \033[33mHELP\033[37m                            \xBA\n"
-	        "\xBA                                                                    \xBA\n"
-	        "\xBA To adjust the emulated CPU speed, use \033[31m%s+F11\033[37m and \033[31m%s+F12\033[37m.%s%s       \xBA\n"
-	        "\xBA To activate the keymapper \033[31m%s+F1\033[37m.%s                                 \xBA\n"
-	        "\xBA For more information read the \033[36mREADME\033[37m file in the DOSBox directory. \xBA\n"
-	        "\xBA                                                                    \xBA\n");
+	        "[bgcolor=blue][color=white]╔════════════════════════════════════════════════════════════════════╗\n"
+	        "║ [color=green]Welcome to DOSBox Staging %-40s[color=white] ║\n"
+	        "║                                                                    ║\n"
+	        "║ For a short introduction for new users type: [color=yellow]INTRO[color=white]                 ║\n"
+	        "║ For supported shell commands type: [color=yellow]HELP[color=white]                            ║\n"
+	        "║                                                                    ║\n"
+	        "║ To adjust the emulated CPU speed, use [color=red]%s+F11[color=white] and [color=red]%s+F12[color=white].%s%s       ║\n"
+	        "║ To activate the keymapper [color=red]%s+F1[color=white].%s                                 ║\n"
+	        "║ For more information read the [color=cyan]README[color=white] file in the DOSBox directory. ║\n"
+	        "║                                                                    ║\n");
 	MSG_Add("SHELL_STARTUP_CGA",
-	        "\xBA DOSBox supports Composite CGA mode.                                \xBA\n"
-	        "\xBA Use \033[31mF12\033[37m to set composite output ON, OFF, or AUTO (default).        \xBA\n"
-	        "\xBA \033[31mF10\033[37m selects the CGA settings to change and \033[31m(%s+)F11\033[37m changes it.   \xBA\n"
-	        "\xBA                                                                    \xBA\n");
+	        "║ DOSBox supports Composite CGA mode.                                ║\n"
+	        "║ Use [color=red]F12[color=white] to set composite output ON, OFF, or AUTO (default).        ║\n"
+	        "║ [color=red]F10[color=white] selects the CGA settings to change and [color=red](%s+)F11[color=white] changes it.   ║\n"
+	        "║                                                                    ║\n");
 	MSG_Add("SHELL_STARTUP_CGA_MONO",
-	        "\xBA Use \033[31mF11\033[37m to cycle through green, amber, white and paper-white mode, \xBA\n"
-	        "\xBA and \033[31m%s+F11\033[37m to change contrast/brightness settings.                \xBA\n");
+	        "║ Use [color=red]F11[reset] to cycle through green, amber, white and paper-white mode, ║\n"
+	        "║ and [color=red]%s+F11[reset] to change contrast/brightness settings.                ║\n");
 	MSG_Add("SHELL_STARTUP_HERC",
-	        "\xBA Use \033[31mF11\033[37m to cycle through white, amber, and green monochrome color. \xBA\n"
-	        "\xBA                                                                    \xBA\n");
+	        "║ Use [color=red]F11[reset]to cycle through white, amber, and green monochrome color. ║\n"
+	        "║                                                                    ║\n");
 	MSG_Add("SHELL_STARTUP_DEBUG",
-	        "\xBA Press \033[31m%s+Pause\033[37m to enter the debugger or start the exe with \033[33mDEBUG\033[37m. \xBA\n"
-	        "\xBA                                                                    \xBA\n");
+	        "║ Press [color=red]%s+Pause[color=white] to enter the debugger or start the exe with [color=yellow]DEBUG[color=white]. ║\n"
+	        "║                                                                    ║\n");
 	MSG_Add("SHELL_STARTUP_END",
-	        "\xBA \033[33mhttps://dosbox-staging.github.io\033[37m                                   \xBA\n"
-	        "\xC8\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD"
-	        "\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD"
-	        "\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xCD\xBC\033[0m\n"
+	        "║ [color=yellow]https://dosbox-staging.github.io[color=white]                                   ║\n"
+	        "╚════════════════════════════════════════════════════════════════════╝[reset]\n"
 	        "\n");
 
-	MSG_Add("SHELL_STARTUP_SUB","[color=green]" CANONICAL_PROJECT_NAME " %s\033[0m\n");
+	MSG_Add("SHELL_STARTUP_SUB","[color=green]" CANONICAL_PROJECT_NAME " %s[reset]\n");
 	MSG_Add("SHELL_CMD_CHDIR_HELP","Displays or changes the current directory.\n");
 	MSG_Add("SHELL_CMD_CHDIR_HELP_LONG",
 	        "Usage:\n"
@@ -1196,24 +1200,6 @@ void SHELL_Init() {
 	        "Examples:\n"
 	        "  [color=green]help[reset] [color=cyan]dir[reset]\n"
 	        "  [color=green]help[reset] /all\n");
-	MSG_Add("SHELL_CMD_INTRO_HELP",
-	        "Displays a full-screen introduction to DOSBox Staging.\n");
-	MSG_Add("SHELL_CMD_INTRO_HELP_LONG",
-	        "Usage:\n"
-	        "  [color=green]intro[reset]\n"
-	        "  [color=green]intro[reset] [color=white]PAGE[reset]\n"
-	        "\n"
-	        "Where:\n"
-	        "  [color=white]PAGE[reset] is the page name to display, including [color=white]cdrom[reset], [color=white]mount[reset], and [color=white]special[reset].\n"
-	        "\n"
-	        "Notes:\n"
-	        "  Running [color=green]intro[reset] without an argument displays one information page at a time;\n"
-	        "  press any key to move to the next page. If a page name is provided, then the\n"
-	        "  specified page will be displayed directly.\n"
-	        "\n"
-	        "Examples:\n"
-	        "  [color=green]intro[reset]\n"
-	        "  [color=green]intro[reset] [color=white]cdrom[reset]\n");
 	MSG_Add("SHELL_CMD_MKDIR_HELP", "Creates a directory.\n");
 	MSG_Add("SHELL_CMD_MKDIR_HELP_LONG",
 	        "Usage:\n"
@@ -1476,13 +1462,13 @@ void SHELL_Init() {
 			"Displays or changes file attributes.\n");
 	MSG_Add("SHELL_CMD_ATTRIB_HELP_LONG",
 	        "Usage:\n"
-	        "  \033[32;1mattrib\033[0m \033[37;1m[ATTRIBUTES]\033[0m \033[36;1mPATTERN\033[0m [/S]\n"
+	        "  [color=green]attrib[reset] [color=white][ATTRIBUTES][reset] [color=cyan]PATTERN[reset] [/S]\n"
 	        "\n"
 	        "Where:\n"
-	        "  \033[37;1mATTRIBUTES\033[0m are attributes to apply, including one or more of the following:\n"
-	        "             \033[37;1m+R\033[0m, \033[37;1m-R\033[0m, \033[37;1m+A\033[0m, \033[37;1m-A\033[0m, \033[37;1m+S\033[0m, \033[37;1m-S\033[0m, \033[37;1m+H\033[0m, \033[37;1m-H\033[0m\n"
+	        "  [color=white]ATTRIBUTES[reset] are attributes to apply, including one or more of the following:\n"
+	        "             [color=white]+R[reset], [color=white]-R[reset], [color=white]+A[reset], [color=white]-A[reset], [color=white]+S[reset], [color=white]-S[reset], [color=white]+H[reset], [color=white]-H[reset]\n"
 	        "             Where: R = Read-only, A = Archive, S = System, H = Hidden\n"
-	        "  \033[36;1mPATTERN\033[0m    can be either an exact filename or an inexact filename with\n"
+	        "  [color=cyan]PATTERN[reset]    can be either an exact filename or an inexact filename with\n"
 	        "             wildcards, which are the asterisk (*) and the question mark (?),\n"
 	        "             or an exact name of a directory."
 	        "\n"
@@ -1491,8 +1477,8 @@ void SHELL_Init() {
 	        "  If not specified, the command shows the current file/directory attributes.\n"
 	        "\n"
 	        "Examples:\n"
-	        "  \033[32;1mattrib\033[0m \033[36;1mfile.txt\033[0m\n"
-	        "  \033[32;1mattrib\033[0m \033[37;1m+R\033[0m \033[37;1m-A\033[0m \033[36;1m*.txt\033[0m\n");
+	        "  [color=green]attrib[reset] [color=cyan]file.txt[reset]\n"
+	        "  [color=green]attrib[reset] [color=white]+R[reset] [color=white]-A[reset] [color=cyan]*.txt[reset]\n");
 	MSG_Add("SHELL_CMD_ATTRIB_GET_ERROR", "Unable to get attributes: %s\n");
 	MSG_Add("SHELL_CMD_ATTRIB_SET_ERROR", "Unable to set attributes: %s\n");
 	MSG_Add("SHELL_CMD_CHOICE_HELP",
